;**************************************************************************
;
;                     EduKitBeta & MU Beta example : Blinking
;
; Processor : PIC16F84A
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on A4 LED on S3
;**************************************************************************
; Assembler directives :
        list    p = PIC16F84A   ; processor type
        __config 0x3FF1         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
indf    equ     0x00            ;indirect addressing
status  equ     0x03            ;status is on the 0x03 address
fsr     equ     0x04            ;file select register
porta   equ     0x05
portb   equ     0x06
intcon  equ     0x0B
; General Purpose Register definition :
intcnt  equ     0x10            ;used for interrupt counting
bckwreg equ     0x11            ;working register backup during interrupt
; Destination definition :
w       equ     0x00
f       equ     0x01
; Bits definition :
#define LED     porta,4         ;LED is on the RA4 pin
#define PB      portb,7         ;pushbutton 2 is on the RB7 pin
#define T0IF    intcon,2        ;TMR0 interrupt flag
#define GIE     intcon,7        ;Global Interrupt Enable bit
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     T0IF            ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_B           ;then return
;
        movlw   0x04            ;else reinitialise intcnt register
        movwf   intcnt
	btfsc   LED             ;and change LED state
        goto    Int_A
        bsf     LED
        goto    Int_B
Int_A:  bcf     LED
Int_B:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0x85            ;0x85 = trisa address
        movwf   fsr
        movlw   B'11101111'     ;pin RA4 is output
        movwf   indf            ;write to trisa
        movlw   0x86            ;0x86 = trisb address
        movwf   fsr
        movlw   B'11111111'     ;portb pins are inputs
        movwf   indf            ;write to trisb
        movlw   0x81            ;0x81 = option_register address
        movwf   fsr
        movlw   B'11010111'     ;prescaler assigned to TMR0
        movwf   indf            ;write to option_register
;
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        movlw   0x04            ;initialise intcnt register
        movwf   intcnt
        bsf     GIE             ;enable global interrupt
;
Main:   btfsc   PB              ;test pushbutton
        goto    Main
        bcf     GIE             ;if pressed, disable interrupt
Main_B: btfss   PB              ;wait for release
        goto    Main_B
        bsf     GIE             ;enable interrupt
        goto    Main            ;closes the loop
;**************************************************************************
; end of EduKitBeta & MU Beta example : Blinking
        end
