;**************************************************************************
;
;               EduKitBeta & MU Beta example : A/D Converter 2
;
; Processor : PIC16C710
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on B4 DISPLAY on S4
; 3) Switch on A0 TRIM1 on S3
;**************************************************************************
; Assembler directives :
        list    p = PIC16C710   ; processor type
        __config 0x3FF9         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16c710.INC>
; General Purpose Register definition :
bckwreg equ     0x10            ;working register backup during interrupt
intcnt  equ     0x11            ;interrupt counting
disp1   equ     0x12            ;digit '1'
disp2   equ     0x13            ;digit '10'
disp3   equ     0x14            ;digit '100'
disp4   equ     0x15            ;digit '1000'
rot1    equ     0x16            ;display data
rot2    equ     0x17            ;display data
rot3    equ     0x18            ;display data
rot4    equ     0x19            ;display data
cntrot  equ     0x1A            ;display segment counting
param0  equ     0x1B            ;used in Wto5d subroutine
param1  equ     0x1C            ;used in Wto5d subroutine
param2  equ     0x1D            ;used in Wto5d subroutine
param3  equ     0x1E            ;used in Wto5d subroutine
param4  equ     0x1F            ;used in Wto5d subroutine
count   equ     0x20            ;used in Wto5d subroutine
temp    equ     0x21            ;used in Wto5d subroutine
addata  equ     0x22            ;A/D converter result
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_A           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        bsf     adcon0,GO       ;start A/D conversion
Adw:    btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw
        movf    adres,W         ;copy adres to addata
        movwf   addata
Int_A:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0x00
        movwf   portb           ;clear portb
        bsf     status,RP0      ;bank 1
        movlw   B'00000010'     ;pin RA0 is analog input
        movwf   adcon1
        movlw   B'00101111'     ;pins RB4,6,7 are outputs
        movwf   trisb
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        clrf    addata
        movlw   B'01000001'     ;enable A/D converter module
        movwf   adcon0
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   movf    addata,W        ;read new data
        movwf   param0
        clrf    param1
        call    Wto5d           ;convert to decimal number
        movf    param2,W        ;save result to disp1-disp4
        andlw   0x0F
        movwf   disp1
        swapf   param2,W
        andlw   0x0F
        movwf   disp2
        movf    param3,W
        andlw   0x0F
        movwf   disp3
        swapf   param3,W
        andlw   0x0F
        movwf   disp4
        call    Display         ;display data
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A  rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;
Wto5d:  bcf     status,C        ;this subroutine converts 16-bit number
        clrf    count           ;in param0(LSB) and param1(MSB) to
        bsf     count,4         ;5-digit decimal number in param2-4
        clrf    param4
        clrf    param3
        clrf    param2
Wto5d2: rlf     param0,F
        rlf     param1,F
        rlf     param2,F
        rlf     param3,F
        rlf     param4,F
        decf    count,F
        btfsc   status,Z
        return
        movf    param2,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param2
        movf    param2,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
	movwf   param2
        movf    param3,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param3
        movf    param3,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param3
        movf    param4,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param4
        movf    param4,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param4
        goto    Wto5d2
;**************************************************************************
; end of EduKitBeta & MU Beta example : A/D Converter 2
        end
