;**************************************************************************
;
;               EduKitBeta & MU Beta example : Comparator
;
; Processor : PIC16C710
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on B4 DISPLAY on S4
; 3) Switch on A0 TRIM1 on S3
; 3) Switch on A1 TRIM2 on S3
;**************************************************************************
; Assembler directives :
        list    p = PIC16C710   ; processor type
        __config 0x3FF9         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16c710.INC>
; General Purpose Register definition :
bckwreg equ     0x10            ;working register backup during interrupt
intcnt  equ     0x11            ;interrupt counting
disp1   equ     0x12            ;digit '1'
disp2   equ     0x13            ;digit '10'
disp3   equ     0x14            ;digit '100'
disp4   equ     0x15            ;digit '1000'
rot1    equ     0x16            ;display data
rot2    equ     0x17            ;display data
rot3    equ     0x18            ;display data
rot4    equ     0x19            ;display data
cntrot  equ     0x1A            ;display segment counting
bcksta  equ     0x1B            ;status register backup during interrrupt
tmp     equ     0x1C            ;first A/D result
cnt1    equ     0x1D            ;used in Wait subroutine
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Intrt           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        bsf     adcon0,GO       ;start A/D conversion
Adw1:   btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw1
        movf    adres,W         ;copy adres to tmp
        movwf   tmp
        bsf     adcon0,CHS0     ;analog channel = RA1
        call    Wait            ;A/D acquisition time
        bsf     adcon0,GO       ;start A/D conversion
Adw2:   btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw2
        bcf     adcon0,CHS0     ;analog channel = RA0
        movf    adres,W         ;compare tmp with adres
        subwf   tmp,W           ;tmp - adres
        btfsc   status,Z
        goto    Dieq            ;equal
        btfss   status,C
        goto    Dilo            ;tmp < adres
        goto    Dihi            ;tmp > adres
;
Dieq:   movlw   0x04            ;equal
        movwf   disp2
        movwf   disp1
        goto    Intrt 
Dihi:   movlw   0x00            ;Higher
        movwf   disp2
        addlw   0x02
        movwf   disp1
        goto    Intrt
Dilo:   movlw   0x01            ;lower
        movwf   disp2
        addlw   0x02
        movwf   disp1
Intrt:  swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0x00
        movwf   portb           ;clear portb
        bsf     status,RP0      ;bank 1
        movlw   B'00000010'     ;pins RA0,1 are analog inputs
        movwf   adcon1
        movlw   B'00101111'     ;pins RB4,6,7 are outputs
        movwf   trisb
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        movlw   0x05
        movwf   disp1
        movwf   disp2
        movwf   disp3
        movwf   disp4
        movlw   B'01000001'     ;enable A/D converter module
        movwf   adcon0
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        call    Wait
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   call    Display         ;display data
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'10001001'     ;H
        retlw   B'11000111'     ;L
        retlw   B'11101111'     ;i
        retlw   B'10100011'     ;o
        retlw   B'10111111'     ;-
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A  rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;**************************************************************************
; end of EduKitBeta & MU Beta example : Comparator
        end
