;**************************************************************************
;
;           EduKitBeta & MU Beta example : Serial EEPROM 24xx (SW)
;
; Processor : PIC16F627
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Insert 24LC01B into socket
; 2) Switch off all DIP switches except:
; 3) Switch on A4 LED on S3
; 4) Switch on B0 I2C_CLK on S4
; 5) Switch on B5 I2C_DATA on S4
; 6) Switch on B4 DISPLAY on S4
;**************************************************************************
; Assembler directives :
        list    p = PIC16F627   ; processor type
        __config 0x3F61         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f627.INC>
; General Purpose Register definition :
intcnt  equ     0x20            ;interrupt counting
disp1   equ     0x21            ;digit '1'
disp2   equ     0x22            ;digit '10'
disp3   equ     0x23            ;digit '100'
disp4   equ     0x24            ;digit '1000'
disp1t  equ     0x25            ;store disp1 value
disp2t  equ     0x26            ;store disp2 value
disp3t  equ     0x27            ;store disp3 value
disp4t  equ     0x28            ;store disp4 value
rot1    equ     0x29            ;display data
rot2    equ     0x2A            ;display data
rot3    equ     0x2B            ;display data
rot4    equ     0x2C            ;display data
cntrot  equ     0x2D            ;display segment counting
eedatb  equ     0x2E            ;EEPROM data write/read
intmp   equ     0x2F            ;used in Input subroutine
eeadrs  equ     0x30            ;used in Wree and Rdee subroutines
swrtmp  equ     0x31            ;used in Swree subroutine
buffer  equ     0x32            ;used in Output and Input subroutines
bufcnt  equ     0x33            ;used in Output and Input subroutines
bckwreg equ     0x70            ;working register backup during interrupt
bcksta  equ     0x71            ;status register backup during interrupt
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
#define SDA     portb,5         ;I2C data
#define SCL     portb,0         ;I2C clock
#define LED     porta,4         ;LED
#define PB2     portb,7         ;pushbutton 2
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_D           ;then return
;
        movlw   0x19            ;else reinitialise intcnt register
        movwf   intcnt
        incf    disp1,f         ;increment disp1
        movlw   0x0A
        subwf   disp1,w         ;if disp1 > 9 
        btfss   status,C
        goto    Int_A
        clrf    disp1           ;then disp1 = 0
        incf    disp2,f         ;and increment disp2
Int_A:  movlw   0x0A
        subwf   disp2,w         ;if disp2 > 9 
        btfss   status,C
        goto    Int_B
        clrf    disp2           ;then disp2 = 0
        incf    disp3,f         ;and increment disp3
Int_B:  movlw   0x0A
        subwf   disp3,w         ;if disp3 > 9 
        btfss   status,C
        goto    Int_C
        clrf    disp3           ;then disp3 = 0
        incf    disp4,f         ;and increment disp4
Int_C:  movlw   0x0A
        subwf   disp4,w         ;if disp4 > 9 then disp4 = 0
        btfsc   status,C
        clrf    disp4
Int_D:  swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   B'00000000'     ;clear portb latch
        movwf   portb
        movlw   B'00010000'     ;set RA4 pin latch
        movwf   porta
        bsf     status,RP0      ;bank 1
        movlw   B'00101111'     ;pins RB7,6,4 are outputs
        movwf   trisb
        movlw   B'11101111'     ;pin RA4 is output
        movwf   trisa
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
;
        movlw   0x00            ;read address 00
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   disp4           ;save to disp4
        movlw   0x01            ;read address 01
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   disp3            ;save to disp3
        movlw   0x02            ;read address 02
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   disp2            ;save to disp2
        movlw   0x03            ;read address 03
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   disp1            ;save to disp1
;
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
        movlw   0x86            ;0x86 = trisb address
        movwf   fsr
;
Main:   call    Display
        bsf     LED
;
        bsf     indf,7          ;RB7 is input
        nop                     ;delay to stable inputs
        nop
        nop
        nop
        nop
        btfss   PB2             ;if pushbutton is pressed
        bcf     LED             ;turn on the LED
        btfsc   PB2
        bsf     LED
        bcf     indf,7          ;RB7 is output
;
        btfsc   LED             ;if the LED is on
        goto    Main
        bcf     intcon,GIE      ;disable interrupt
        movf    disp1,w         ;save disp1-disp4 state
        movwf   disp1t
        movf    disp2,w
        movwf   disp2t
        movf    disp3,w
        movwf   disp3t
        movf    disp4,w
        movwf   eedatb
        bsf     intcon,GIE      ;enable interrupt
        movf    portb,W         ;clear RB0,5 pin latch
        andlw   B'11011110'
        movwf   portb
        movlw   0x00
        call    Swree           ;write to EEPROM
        movf    disp3t,w
        movwf   eedatb
        movlw   0x01
        call    Swree           ;write to EEPROM
        movf    disp2t,w
        movwf   eedatb
        movlw   0x02
        call    Swree           ;write to EEPROM
        movf    disp1t,w
        movwf   eedatb
        movlw   0x03
        call    Swree           ;write to EEPROM
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F           ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A  rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;
Setda:  bsf     status,RP0      ;sets SDA to 1
        bsf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Clrda:  bsf     status,RP0      ;sets SDA to 0
        bcf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Setcl:  bsf     status,RP0      ;sets SCL to 1
        bsf     SCL
        bcf     status,RP0      ;bank 0
Setcl2: btfss   SCL             ;wait for SCL goes Hi
        goto    Setcl2
        return
;
Clrcl:  bsf     status,RP0      ;sets SCL to 0
        bcf     SCL
        bcf     status,RP0      ;bank 0
        return
;
Swree:  movwf   eeadrs          ;this subroutine writes to EEPROM
        movf    eedatb,w        ;use this subroutine to avoid multiply
        movwf   swrtmp          ;rewriting one address with the same value
        movf    eeadrs,w
        call    Rdee            ;read EEPROM data
        movf    eedatb,w        ;compare data in EEPROM
        xorwf   swrtmp,w        ;with data to write
        btfsc   status,Z
        return
        movf    swrtmp,w
        movwf   eedatb
        movf    eeadrs,w
        call    Wree            ;write to EEPROM
        return
;
Sndsb:  call    Setda           ;this subroutine generates I2C start bit
        nop
        call    Setcl
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrda           ;SDA = 0
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        return
;
Sndpb:  call    Clrcl           ;this subroutine generates I2C stop bit
        nop
        call    Clrda
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setcl           ;SCL = 1
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setda           ;SDA = 1
        nop
        return
;
Output: movwf   buffer          ;place data in buffer
        movlw   0x08
        movwf   bufcnt
Outpt2: btfss   buffer,7        ;SDA := buffer MSb
        call    Clrda
        btfsc   buffer,7
        call    Setda
        call    Setcl           ;SCL = 1
        nop
        call    Clrcl           ;SCL = 0
        nop
        rlf     buffer,f
        decfsz  bufcnt,f
        goto    Outpt2          ;repeat 8x
        call    Setda           ;SDA = 1 (Hi-Impedance)
        call    Setcl           ;SCL = 1
        bcf     buffer,7        ;9.clock pulse
        btfsc   SDA             ;read ACKF
        bsf     buffer,7
        call    Clrcl           ;SCL = 0
        return
;
Input:  call    Setda           ;serial receive 8 bits
        movlw   0x08            ;SDA = 1 (Hi-Impedance)
        movwf   bufcnt
Inpt2:  call    Setcl           ;SCL = 1
        nop
        bcf     status,C
        btfsc   SDA             ;read data from SDA
        bsf     status,C
        rlf     intmp,f
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        nop
        decfsz  bufcnt,f
        goto    Inpt2           ;repeat 8x
        btfss   buffer,7
        call    Clrda           ;ack will be send
        btfsc   buffer,7
        call    Setda           ;ack will not be send
        nop
        call    Setcl           ;SCL = 1 - send ACK
        nop
        nop
        call    Clrcl           ;SCL = 0
        movf    intmp,w
        return
;
Rdee:   movwf   eeadrs          ;read EEPROM subroutine
Rdee2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output
        btfss   buffer,7        ;acknowledge pooling
        goto    Rdee3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        goto    Rdee2
Rdee3:  movf    eeadrs,w
        call    Output          ;write address to EEPROM
        call    Sndsb           ;start bit
        movlw   B'10100001'     ;control byte - read
        call    Output
        bsf     buffer,7        ;select NO_ACK bit
        call    Input           ;read data
        movwf   eedatb
        call    Sndpb           ;stop bit
        return
;
Wree:   movwf   eeadrs          ;write EEPROM subroutine
Wree2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output
        btfss   buffer,7        ;acknowledge pooling
        goto    Wree3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        goto    Wree2
Wree3:  movf    eeadrs,w
        call    Output          ;write address to EEPROM
        movf    eedatb,w
        call    Output          ;write data to EEPROM
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of EduKitBeta & MU Beta example : Serial EEPROM 24xx (SW)
        end
