;**************************************************************************
;
;           EduKitBeta & MU Beta example : Thermometer
;
; Processor : PIC16F627
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on A4 LED on S3
; 3) Switch on B0 I2C_CLK on S4
; 4) Switch on B5 I2C_DATA on S4
; 5) Switch on B4 DISPLAY on S4
;**************************************************************************
; Assembler directives :
        list    p = PIC16F627   ; processor type
        __config 0x3F61         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f627.INC>
; General Purpose Register definition :
disp1   equ     0x20            ;digit '1'
disp2   equ     0x21            ;digit '10'
disp3   equ     0x22            ;digit '100'
disp4   equ     0x23            ;digit '1000'
rot1    equ     0x24            ;display data
rot2    equ     0x25            ;display data
rot3    equ     0x26            ;display data
rot4    equ     0x27            ;display data
cntrot  equ     0x28            ;display segment counting
intmp   equ     0x29            ;used in Input subroutine
buffer  equ     0x2A            ;used in Output and Input subroutines
bufcnt  equ     0x2B            ;used in Output and Input subroutines
temph   equ     0x2C            ;temperature, MSB
templ   equ     0x2D            ;temperature, LSB
cvcnt   equ     0x2E            ;used in H2dec subroutine
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
#define SDA     portb,5         ;I2C data
#define SCL     portb,0         ;I2C clock
#define LED     porta,4         ;LED
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  movlw   B'00000000'     ;clear portb latch
        movwf   portb
        movlw   B'00010000'     ;set RA4 pin latch
        movwf   porta
        bsf     status,RP0      ;bank 1
        movlw   B'00101111'     ;pins RB7,6,4 are outputs
        movwf   trisb
        movlw   B'11101111'     ;pin RA4 is output
        movwf   trisa
        bcf     status,RP0      ;bank 0
        movlw   B'00110001'     ;setup timer1
        movwf   t1con
        movlw   0x0A
        movwf   disp1           ;fixed "C"
;
Main:   btfss   pir1,TMR1IF     ;wait for timer1 overflow
        goto    Main
        bcf     pir1,TMR1IF
        movf    portb,W         ;clear RB0,5 pin latch
        andlw   B'11011110'
        movwf   portb
        call    Rdtmp           ;read temperature
;
        movlw   0x00
        btfsc   templ,7         ; 0 / 0.5 C ?
        movlw   0x05
        movwf   disp2
        bsf     LED             ;switch LED off
        btfss   temph,7         ;temperature below zero ?
        goto    Tc_A            ;no
        bcf     LED             ;yes, switch LED on
        rlf     templ,F         ;convert value in temph:l
        rlf     temph,F
        comf    temph,F
        incf    temph,F
        rrf     temph,F
Tc_A:   movlw   0x7F            ;mask MSb
        andwf   temph,F
        call    H2bcd           ;convert to BCD format
;
        call    Display
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11000110'     ;C
        retlw   B'10111111'     ;-
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
H2bcd:  clrf    disp3           ;convert temph to bcd data
        clrf    disp4
        movf    temph,W
        btfsc   status,Z        ;zero?
        return
        movwf   cvcnt
H2bcd2: incf    disp3,F         ;increment disp3
        movlw   0x0A
        subwf   disp3,W
        btfss   status,C
        goto    H2bcd3
        clrf    disp3
        incf    disp4,F         ;increment disp4
        movlw   0x0A
        subwf   disp4,W
        btfss   status,C
        goto    H2bcd3
        movlw   0x0B            ;overflow
        movwf   disp2
        movwf   disp3
        movwf   disp4
        return
H2bcd3: decfsz  cvcnt,F
        goto    H2bcd2
        return
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        andlw   B'01111111'     ;dot
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A: rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;
Setda:  bsf     status,RP0      ;sets SDA to 1
        bsf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Clrda:  bsf     status,RP0      ;sets SDA to 0
        bcf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Setcl:  bsf     status,RP0      ;sets SCL to 1
        bsf     SCL
        bcf     status,RP0      ;bank 0
Setcl2: btfss   SCL             ;wait for SCL goes Hi
        goto    Setcl2
        return
;
Clrcl:  bsf     status,RP0      ;sets SCL to 0
        bcf     SCL
        bcf     status,RP0      ;bank 0
        return
;
Sndsb:  call    Setda           ;this subroutine generates I2C start bit
        nop
        call    Setcl
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrda           ;SDA = 0
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        return
;
Sndpb:  call    Clrcl           ;this subroutine generates I2C stop bit
        nop
        call    Clrda
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setcl           ;SCL = 1
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setda           ;SDA = 1
        nop
        return
;
Output: movwf   buffer          ;place data in buffer
        movlw   0x08
        movwf   bufcnt
Outpt2: btfss   buffer,7        ;SDA := buffer MSb
        call    Clrda
        btfsc   buffer,7
        call    Setda
        call    Setcl           ;SCL = 1
        nop
        call    Clrcl           ;SCL = 0
        nop
        rlf     buffer,f
        decfsz  bufcnt,f
        goto    Outpt2          ;repeat 8x
        call    Setda           ;SDA = 1 (Hi-Impedance)
        call    Setcl           ;SCL = 1
        bcf     buffer,7        ;9.clock pulse
        btfsc   SDA             ;read ACKF
        bsf     buffer,7
        call    Clrcl           ;SCL = 0
        return
;
Input:  call    Setda           ;serial receive 8 bits
        movlw   0x08            ;SDA = 1 (Hi-Impedance)
        movwf   bufcnt
Inpt2:  call    Setcl           ;SCL = 1
        nop
        bcf     status,C
        btfsc   SDA             ;read data from SDA
        bsf     status,C
        rlf     intmp,f
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        nop
        decfsz  bufcnt,f
        goto    Inpt2           ;repeat 8x
        btfss   buffer,7
        call    Clrda           ;ack will be send
        btfsc   buffer,7
        call    Setda           ;ack will not be send
        nop
        call    Setcl           ;SCL = 1 - send ACK
        nop
        nop
        call    Clrcl           ;SCL = 0
        movf    intmp,w
        return
;
Rdtmp:  call    Sndsb           ;start bit
        movlw   B'10010010'     ;address byte, write
        call    Output
        movlw   B'00000000'     ;pointer = 00 = TEMP
        call    Output
        call    Sndsb           ;restart bit
        movlw   B'10010011'     ;address byte, read
        call    Output
        bcf     buffer,7        ;select ACK bit
        call    Input           ;read data
        movwf   temph
        bsf     buffer,7        ;select NO_ACK bit
        call    Input           ;read data
        movwf   templ
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of EduKitBeta & MU Beta example : Thermometer
        end
