;**************************************************************************
;
;           EduKitBeta & MU Beta example : UART - Rx
;
; Processor : PIC16F627
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on B1 UART_RX on S3
; 3) Switch on B4 DISPLAY on S4
;**************************************************************************
; Assembler directives :
        list    p = PIC16F627   ; processor type
        __config 0x3F61         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f627.INC>
; General Purpose Register definition :
disp1   equ     0x20            ;digit '1'
disp2   equ     0x21            ;digit '10'
disp3   equ     0x22            ;digit '100'
disp4   equ     0x23            ;digit '1000'
rot1    equ     0x24            ;display data
rot2    equ     0x25            ;display data
rot3    equ     0x26            ;display data
rot4    equ     0x27            ;display data
cntrot  equ     0x28            ;display segment counting
rcdata  equ     0x29            ;received data
bckwreg equ     0x70            ;working register backup during interrupt
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
Interr: movwf   bckwreg         ;save wreg during interrupt
        movf    rcreg,W         ;read received data
        movwf   rcdata          ;RCIF will be automaticaly cleared
        movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  clrf    portb           ;clear portb latch
        bsf     status,RP0      ;bank 1
        movlw   B'00101111'     ;pins RB7,6,4 are outputs
        movwf   trisb
        movlw   0x14            ;9600 Bd
        movwf   spbrg
        movlw   B'00000110'     ;disable transmit
        movwf   txsta
        movlw   B'00100000'     ;unmask interrupt from UART Rx
        movwf   pie1
        bcf     status,RP0      ;bank 0
        movlw   B'10010000'     ;serial port enabled
        movwf   rcsta
        movlw   B'01000000'     ;unmask peripheral interrupt
        movwf   intcon
        clrf    rcdata
        movf    rcreg,W         ;clear RCIF
        bsf     intcon,GIE      ;enable interrupt
;
        clrf    disp3
        clrf    disp4
Main:   movf    rcdata,W
        movwf   disp2
        andlw   0x0F
        movwf   disp1           ;first digit
        swapf   disp2,W
        andlw   0x0F
        movwf   disp2           ;second digit
        call    Display
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'10001000'     ;A
        retlw   B'10000011'     ;B
        retlw   B'11000110'     ;C
        retlw   B'10100001'     ;D
        retlw   B'10000110'     ;E
        retlw   B'10001110'     ;F
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A: rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;**************************************************************************
; end of EduKitBeta & MU Beta example : UART - Rx
        end
