;**************************************************************************
;
;           EduKitBeta & MU Beta example : Alarm clock
;
; Processor : PIC16F627
; Clock : XT 3.2768 MHz
;
; On your EduKitBeta board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on A4 LED on S3
; 3) Switch on B3 PIEZO on S4
; 4) Switch on B4 DISPLAY on S4
;**************************************************************************
; Assembler directives :
        list    p = PIC16F627   ; processor type
        __config 0x3F61         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f627.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in Wait subroutine
cnt2    equ     0x21            ;used in Wait subroutine
bpcnt   equ     0x22            ;beep counting
dstat   equ     0x23            ;display state
btns    equ     0x24            ;pushbuttons state, 0 - 3
ontime  equ     0x25            ;bit 0 = alarm flag
thrsh   equ     0x26            ;real time, 10 * hours, 0 - 2
thrsl   equ     0x27            ;real time, hours, 0 - 9 / 0 - 3
tminh   equ     0x28            ;real time, 10 * minutes 0 - 5
tminl   equ     0x29            ;real time, minutes 0 - 9
tsec    equ     0x2A            ;real time, 1/2 seconds, 0 - 119
tmsec   equ     0x2B            ;real time, 1/400 seconds, 0 - 199
ahrsh   equ     0x2C            ;alarm time, 10 * hours, 0 - 2
ahrsl   equ     0x2D            ;alarm time, hours, 0 - 9 / 0 - 3
aminh   equ     0x2E            ;alarm time, 10 * minutes, 0 - 5
aminl   equ     0x2F            ;alarm time, minutes, 0 - 9
disp1   equ     0x30            ;digit '1'
disp2   equ     0x31            ;digit '10'
disp3   equ     0x32            ;digit '100'
disp4   equ     0x33            ;digit '1000'
rot1    equ     0x34            ;display data
rot2    equ     0x35            ;display data
rot3    equ     0x36            ;display data
rot4    equ     0x37            ;display data
cntrot  equ     0x38            ;display segment counting
bcksta  equ     0x70            ;status register backup during interrupt
bckwreg equ     0x71            ;working register backup during interrupt
; Bits definition :
#define DATA    portb,7         ;shift register data
#define CLK     portb,4         ;shift register clock
#define STR     portb,6         ;shift register strobe
#define LED     porta,4         ;LED is on the RA4 pin
#define PB2     portb,7         ;pushbutton 2
#define PB3     porta,4         ;pushbutton 3
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x04            ;interrupt vector
Interr: movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
;
        incf    tmsec,F         ;increment time
        movlw   0x19            ;1/50 second
        subwf   tmsec,W
        movlw   0x00
        btfsc   status,Z
        movwf   tmsec
        btfsc   status,Z
        incf    tsec,F
        movlw   0x78            ;1/2 second
        subwf   tsec,W
        movlw   0x00
        btfsc   status,Z
        movwf   tsec
        btfsc   status,Z
        incf    tminl,F
        movlw   0x0A            ;minutes
        subwf   tminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminl
        btfsc   status,Z
        incf    tminh,F
        movlw   0x06            ;10 * minutes
        subwf   tminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminh
        btfsc   status,Z
        incf    thrsl,F
        movlw   0x0A            ;hours
        btfsc   thrsh,1         ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   thrsl
        btfsc   status,Z
        incf    thrsh,F
        movlw   0x03            ;10 * hours
        subwf   thrsh,W
        btfsc   status,Z
        clrf    thrsh
;
        movf    tmsec,F         ;test if real time = alarm time
        btfss   status,Z
        goto    Rdbt
        movf    tsec,F          ;tmsec = 0
        btfss   status,Z
        goto    Rdbt
        movf    tminl,W         ;tsec = 0
        xorwf   aminl,W         ;test if tminl = aminl
        btfss   status,Z
        goto    Rdbt
        movf    tminh,W         ;test if tminh = aminh
        xorwf   aminh,W
        btfss   status,Z
        goto    Rdbt
        movf    thrsl,W         ;test if thrsl = ahrsl
        xorwf   ahrsl,W
        btfss   status,Z
        goto    Rdbt
        movf    thrsh,W         ;test if thrsh = ahrsh
        xorwf   ahrsh,W
        btfss   status,Z
        goto    Rdbt
        bsf     ontime,0        ;if real_time=alarm_time then set flag
;
Rdbt:   clrf    btns            ;read pushbuttons state
        bsf     status,RP0      ;bank 1
        bsf     LED             ;RA4 is input
        bsf     DATA            ;RB7 is input
        bcf     status,RP0      ;bank 0
        nop                     ;delay to stable inputs
        nop
        nop
        nop
        nop
        btfss   PB2             ;if pushbutton is pressed
        bsf     btns,0          ;set flag
        btfss   PB3
        bsf     btns,1
        bsf     status,RP0      ;bank 1
        bcf     LED             ;RA4 is output
        bcf     DATA            ;RB7 is output
        bcf     status,RP0      ;bank 0
;
        movf    tmsec,F         ;test if tmsec = 0
        btfss   status,Z
        goto    Intret          ;no, return
        movf    dstat,W         ;yes, refresh display
        andlw   0x07
        addwf   pcl,f
        goto    Ds0
        goto    Ds1
        goto    Ds2
        goto    Ds3
        goto    Ds4
        goto    Ds5
        goto    Start           ;error -> reset
        goto    Start           ;error -> reset
;
Ds0:    movf    thrsh,W         ;show real time
        movwf   disp4
        movf    thrsl,W
        btfsc   tsec,0
        iorlw   0x80            ;blinking dot
        movwf   disp3
        movf    tminh,W
        movwf   disp2
        movf    tminl,W
        movwf   disp1
        bsf     LED             ;switch LED off
        call    Display
        goto    Intret
;
Ds1:    movf    ahrsh,W         ;show alarm time
        movwf   disp4
        movf    ahrsl,W
        iorlw   0x80            ;dot
        movwf   disp3
        movf    aminh,W
        movwf   disp2
        movf    aminl,W
        movwf   disp1
        bcf     LED             ;switch LED on
        call    Display
        goto    Intret
;
Ds2:    movf    thrsh,W         ;setup hours (real time)
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp4
        movf    thrsl,W
        btfsc   tsec,0
        iorlw   0x80            ;blinking dot
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp3
        movf    tminh,W
        movwf   disp2
        movf    tminl,W
        movwf   disp1
        bsf     LED             ;switch LED off
        call    Display
        goto    Intret
;
Ds3:    movf    thrsh,W         ;setup minutes (real time)
        movwf   disp4
        movf    thrsl,W
        btfsc   tsec,0
        iorlw   0x80            ;blinking dot
        movwf   disp3
        movf    tminh,W
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp2
        movf    tminl,W
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp1
        bsf     LED             ;switch LED off
        call    Display
        goto    Intret
;
Ds4:    movf    ahrsh,W         ;setup hours (alarm time)
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp4
        movf    ahrsl,W
        iorlw   0x80            ;dot
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp3
        movf    aminh,W
        movwf   disp2
        movf    aminl,W
        movwf   disp1
        bcf     LED             ;switch LED on
        call    Display
        goto    Intret
;
Ds5:    movf    ahrsh,W         ;setup minutes (alarm time)
        movwf   disp4
        movf    ahrsl,W
        iorlw   0x80            ;dot
        movwf   disp3
        movf    aminh,W
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp2
        movf    aminl,W
        btfss   tsec,0
        movlw   0x0A            ;blinking digit
        movwf   disp1
        bcf     LED             ;switch LED on
        call    Display
        goto    Intret
;
Intret: swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Display bcf     STR             ;clear shift register strobe signal
        movf    disp1,W         ;get display segment values
        call    Table
        movwf   rot1
        movf    disp2,W
        call    Table
        movwf   rot2
        movf    disp3,W
        call    Table
        btfsc   disp3,7
        andlw   B'01111111'     ;dot
        movwf   rot3
        movf    disp4,W
        call    Table
        movwf   rot4
        movlw   d'32'           ;loop 32 times (4 digits with 8 segments)
        movwf   cntrot
Disp_A: rlf     rot1,f          ;32-bit register rotation
        rlf     rot2,f
        rlf     rot3,f
        rlf     rot4,f
        bsf     DATA            ;set shift register data pin
        btfss   status,C
        bcf     DATA
        bcf     CLK             ;clock pulse
        nop
        nop
        bsf     CLK             ;rising edge = write data
        decfsz  cntrot,f        ;end of the loop?
        goto    Disp_A          ;no, loop again
        bsf     STR             ;set shift register strobe to display
        return
;
Wait:   movlw   0x60            ;this subroutine wait 74021 cycles
        movwf   cnt1            ;(90ms)
Wait_B: clrf    cnt2
Wait_C: decfsz  cnt2,f          ;decrement cnt2
        goto    Wait_C
        decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;--------------------------------------------------------------------------
Start:  movlw   B'00000000'     ;clear portb latch
        movwf   portb
        movlw   B'00010000'     ;set RA4 pin latch
        movwf   porta
        clrf    ccpr1h          ;PWM duty cycle = 0%
        clrf    ccpr1l
        movlw   B'00001111'     ;CCP1 mode = PWM
        movwf   ccp1con
        clrf    tmr2
        movlw   B'00000100'     ;timer2 on, prescaler 1:1
        movwf   t2con
        bsf     status,RP0      ;bank 1
        movlw   B'11101111'     ;pin RA4 is output
        movwf   trisa
        movlw   B'00100111'     ;pins RB7,6,4,3 are outputs
        movwf   trisb
        movlw   B'11111111'     ;PWM period
        movwf   pr2
        movlw   B'11010101'     ;prescaler assigned to TMR0
        movwf   option_reg      ;TMR0 overflow period = 20ms
        bcf     status,RP0      ;bank 0
;
        clrf    thrsh           ;start time 00:00:00.000
        clrf    thrsl
        clrf    tminh
        clrf    tminl
        clrf    tsec
        clrf    tmsec
        movlw   0x01            ;alarm time 12:00
        movwf   ahrsh
        movlw   0x02
        movwf   ahrsl
        clrf    aminh
        clrf    aminl
        clrf    dstat
        movlw   0x00
        movwf   btns
        clrf    ontime
;
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   btfsc   btns,0          ;test pushbutton 2
        goto    Btn2
        btfsc   btns,1          ;test pushbutton 3
        goto    Btn3
        btfss   ontime,0        ;test ontime flag
        goto    Main            ;closes the loop
;
        movlw   0x4B            ;Alarm
        movwf   bpcnt
        bcf     ontime,0
Alarm:  movlw   0x80            ;PWM on
        movwf   ccpr1l
        call    Wait
        clrf    ccpr1l          ;PWM off
        call    Wait
        call    Wait
        movf    btns,F          ;test pushbuttons
        btfss   status,Z
        goto    Alarm2
        decfsz  bpcnt,F
        goto    Alarm
Alarm2: movf    btns,F          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Alarm2
        goto    Main
;--------------------------------------------------------------------------
Btn2:   movlw   0x01            ;change display state
        xorwf   dstat,W
        btfss   status,Z        ;if dstat = 1
        goto    Btn2_A
        clrf    dstat           ;then dstat = 0
        goto    Btn2_D
Btn2_A: movlw   0x05
        xorwf   dstat,W
        btfss   status,Z        ;if dstat = 5
        goto    Btn2_C
        clrf    dstat           ;then dstat = 0
        goto    Btn2_D
Btn2_C: incf    dstat,F         ;else increment dstat
Btn2_D: movf    btns,F          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn2_D
        goto    Main
;
Btn3:   movlw   0x02            ;test dstat
        subwf   dstat,W
        btfsc   status,c        ;if dstat = 0 or 1
        goto    Btn3_A
        movlw   0x02            ;then dstat = 2
        movwf   dstat
        goto    Btn3_B
;
Btn3_A: bcf     intcon,GIE      ;else disable interrupt
        movlw   0x02
        xorwf   dstat,W
        btfsc   status,Z        ;if dstat = 2
        goto    Incth           ;then increment real time hours
        movlw   0x03
        xorwf   dstat,W
        btfsc   status,Z        ;if dstat = 3
        goto    Inctm           ;then increment real time minutes
        movlw   0x04
        xorwf   dstat,W
        btfsc   status,Z        ;if dstat = 4
        goto    Incah           ;then increment alarm time hours
        goto    Incam           ;else increment alarm time minutes
;
Incth:  incf    thrsl,F         ;increment real time
        movlw   0x0A            ;hours
        btfsc   thrsh,1         ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   thrsl
        btfsc   status,Z
        incf    thrsh,F
        movlw   0x03            ;10 * hours
        subwf   thrsh,W
        btfsc   status,Z
        clrf    thrsh
        goto    Btn3_B
;
Inctm:  incf    tminl,F         ;increment real time
        movlw   0x0A            ;minutes
        subwf   tminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminl
        btfsc   status,Z
        incf    tminh,F
        movlw   0x06            ;10 * minutes
        subwf   tminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminh
        goto    Btn3_B
;
Incah:  incf    ahrsl,F         ;increment alarm time
        movlw   0x0A            ;hours
        btfsc   ahrsh,1         ;if ahrsh = 2 then ahrsl = 0 .. 3
        movlw   0x04
        subwf   ahrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   ahrsl
        btfsc   status,Z
        incf    ahrsh,F
        movlw   0x03            ;10 * hours
        subwf   ahrsh,W
        btfsc   status,Z
        clrf    ahrsh
        goto    Btn3_B
;
Incam:  incf    aminl,F         ;increment alarm time
        movlw   0x0A            ;minutes
        subwf   aminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   aminl
        btfsc   status,Z
        incf    aminh,F
        movlw   0x06            ;10 * minutes
        subwf   aminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   aminh
        goto    Btn3_B
;
Btn3_B: bsf     intcon,GIE      ;enable interrupt
        movf    btns,F          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn3_B
        goto    Main
;**************************************************************************
; end of EduKitBeta & MU Beta example : Alarm clock
        end
