/*============================================================================

                  EduKitBeta Example : Blinking

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F84A
 Clock : XT 3.2768 MHz

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on A4 LED


============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit button @ PORTBIT(PORTB,7);
static bit led @ PORTBIT(PORTA,4);

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}


int main(void)
{
  TRISA=0xEF;
  /* initialisation of timer0 */
	
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
  GIE=1;                   /* interrupts are actived */
  period=25;               /* speed */
  /*--------------------------*/


  for(;;) {
    if (button==0)     /* test button */
      period=12;
    else
      period=25;
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      led=!led;
    }
  }
}
