/*============================================================================

                  EduKitBeta Example : Counter

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F84A
 Clock : XT 3.2768 MHz

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on B4 DISPLAY on S4

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & PWRTEN & WDTDIS & XT);


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit data @ PORTBIT(PORTB,7); /* shift register data */
static bit clk @ PORTBIT(PORTB,4); /* shift register clock */
static bit str @ PORTBIT(PORTB,6); /* shift register strobe */

char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count=0;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}

void sendbyte(unsigned char b)
{
  char i;
  char temp;

  temp=b;
  for(i=8;i;i--) {
    if(temp&0x80)
      data=1;
    else
      data=0;
    clk=0;
    asm("nop");
    clk=1;
    temp<<=1;
  }
}

void display(int val)
{
  int i;
  int k;

  str=0;
  k=1000;
  for(i=4;i;i--) {
    sendbyte(segment[(count/k)%10]);
    k/=10;
  }
  str=1;
}

void main(void)
{
  TRISB=0x2F;
  /* initialisation of timer0 */
	
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
  GIE=1;                   /* interrupts are actived */
  period=50;               /* 1 second */
  /*--------------------------*/

  for(;;) {
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      if(++count>9999)     /*counting*/
        count=0;
    }
    display(count);
  } // end of the for loop
}
