/*============================================================================

           EduKitBeta & MU Beta example : Thermometer

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : XT 3.2768 MHz

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on A4 LED on S3
 3) Switch on B0 I2C_CLK on S4
 4) Switch on B5 I2C_DATA on S4
 5) Switch on B4 DISPLAY on S4

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

static bit led @ PORTBIT(PORTA,4);
static bit data @ PORTBIT(PORTB,7); /* shift register data */
static bit clk @ PORTBIT(PORTB,4); /* shift register clock */
static bit str @ PORTBIT(PORTB,6); /* shift register strobe */
static bit i2c_sda @ PORTBIT(PORTB,5);
static bit i2c_scl @ PORTBIT(PORTB,0);
static bit i2c_sda_tris @ PORTBIT(TRISB,5);
static bit i2c_scl_tris @ PORTBIT(TRISB,0);

char segment[12] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90,
                    0xC6,0xBF};

/*============================================================================

       Program

============================================================================*/

void portinit(void)
{
  PORTB=0x00;
  PORTA=0x10;
  TRISB=0x2F;
  TRISA=0xEF;
}

void sendbyte(unsigned char b)
{
  char i;
  char tmp;

  tmp=b;
  for(i=8;i;i--) {
    if(tmp&0x80)
      data=1;
    else
      data=0;
    clk=0;
    asm("nop");
    clk=1;
    tmp<<=1;
  }
}

void display(unsigned int td)
{
  unsigned char hdeg;

  str=0;
  if(td&0x80)
    hdeg=segment[5];
  else
    hdeg=segment[0];
  if(td&0x8000) {
    led=0;  /* temperature below zero, switch led off */
    td^=td;
    td+=0x80;
    td>>=8;
  }
  else {
    led=1;
    td>>=8;
  }
  if(td>99) {
    sendbyte(segment[11]);
    sendbyte(segment[11]);
    sendbyte(segment[11]);
  }
  else {
    sendbyte(segment[(td/10)]);
    sendbyte(segment[td%10]&0x7F);
    sendbyte(hdeg);
  }
  sendbyte(segment[10]);
  str=1;
}

void delayus(unsigned char us)
{
  for (;us--;);
}

void i2c_start_bit(void)
{
  i2c_sda_tris=1;  /* set data bus */
  delayus(1);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
}

void i2c_stop_bit(void)
{
  i2c_scl_tris=0;  /* reset clock bus */
  delayus(1);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=1;  /* set data bus */
}

unsigned char i2c_write(unsigned char data)
/* send byte to i2c device 
   returned value is acknowledge flag */
{ 
  unsigned char mask, flag;

  for(mask=0x80;mask!=0;mask>>=1) {
    if(data&mask)
      i2c_sda_tris=1;
    else
      i2c_sda_tris=0;
    i2c_scl_tris=1;  /* set clock bus */
    asm("nop");
    i2c_scl_tris=0;  /* reset clock bus */
    asm("nop");
  }
  i2c_sda_tris=1;  /* set data bus */
  i2c_scl_tris=1;  /* set clock bus - 9. clock pulse */
  asm("nop");
  flag=i2c_sda; /* read ACKF */
  i2c_scl_tris=0;  /* reset clock bus */
  return flag;
}

unsigned char i2c_read(unsigned char ackdata)
{  /* read byte from i2c bus */
  unsigned char temp, bitcnt;

  i2c_sda_tris=1;  /* set data bus (Hi-Impedance) */
  temp=0x00;
  for(bitcnt=8;bitcnt!=0;bitcnt--) {
    i2c_scl_tris=1;  /* set clock bus */
    temp<<=1;
    temp|=i2c_sda; /* read data */
    asm("nop");
    i2c_scl_tris=0;  /* reset clock bus */
    asm("nop");
  }
  if(ackdata) 
    i2c_sda_tris=1;
  else
    i2c_sda_tris=0;
  i2c_scl_tris=1;  /* set clock bus - send ACK */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
  asm("nop");
  return temp;
}

unsigned int rdtemp(void)
{
  unsigned int tempdata;

  i2c_start_bit();  /* start bit */
  i2c_write(0x92);  /* control byte - write */
  i2c_write(0x00); /* pointer = 00 = TEMP */
  i2c_start_bit();  /* restart bit */
  i2c_write(0x93);  /* control byte - read */
  tempdata=(i2c_read(0)<<8);  /* select ACK bit */
  tempdata|=i2c_read(1);    /* select NO_ACK bit */
  i2c_stop_bit(); /* stop bit */
  return tempdata;
}

void main(void)
{
  unsigned int temp;

  portinit();
  T1CON=0x31;              /* initialise timer1 */

  for(;;) {
    while(!TMR1IF) ;  /* wait for timer1 overflow */
    TMR1IF=0;
    PORTB&=0xDE; /* clear RB0,5 pin latch */
    temp=rdtemp();
    display(temp);

  } // end of the for loop
}
