/*============================================================================

           EduKitBeta & MU Beta example : UART - Tx

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : XT 3.2768 MHz

 Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on B2 UART_TX on S3

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

/*============================================================================

       Program

============================================================================*/

void uart_init(void)
{  /* serial port initialisation */
  RCSTA=0x80; /* pin RC6 is automaticaly configured as output */
  SPBRG=0x14; /* 9600Bd */
  TXSTA=0x26; /* TX enabled */
}

void uart_sendbyte(unsigned char databyte)
{
  TXREG=databyte;
  while(!TRMT) ; /* wait for operation complete */
}

void uart_sendstr(const char *s)
{
  while(*s) {
    uart_sendbyte(*(s++));
  }
}

int main(void)
{
  uart_init();

  uart_sendstr("Hello, World!\r\n");

  for(;;);
}
