/*============================================================================

           EduKitBeta & MU Beta example : UART - Rx

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : XT 3.2768 MHz

 Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on B1 UART_RX on S3
 3) Switch on B4 DISPLAY on S4

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

static bit data @ PORTBIT(PORTB,7); /* shift register data */
static bit clk @ PORTBIT(PORTB,4); /* shift register clock */
static bit str @ PORTBIT(PORTB,6); /* shift register strobe */

const unsigned char segment[]={0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,
                               0x80,0x90,0x88,0x83,0xC6,0xA1,0x86,0x8E};
unsigned char buffer;

/*============================================================================

       Program

============================================================================*/

void portinit(void)
{
  PORTB=0x00;
  TRISB=0x2F;
}

void uart_init(void)
{  /* serial port initialisation */
  RCSTA=0x90; /* receive enabled */
  SPBRG=0x14; /* 9600Bd */
  TXSTA=0x06; /* TX disabled */
  PEIE=1; /* peripheral interrupts enabled */
  RCIE=1;
  buffer=RCREG; /* clear RCIF flag */
  buffer=RCREG;
}

void sendbyte(unsigned char b)
{
  char i;
  char tmp;

  tmp=b;
  for(i=8;i;i--) {
    if(tmp&0x80)
      data=1;
    else
      data=0;
    clk=0;
    asm("nop");
    clk=1;
    tmp<<=1;
  }
}

void display(unsigned char bt)
{
  str=0;
  sendbyte(0xFF);
  sendbyte(0xFF);
  sendbyte(segment[bt>>4]);
  sendbyte(segment[bt&0x0F]);
  str=1;
}

void interrupt uart_rx(void)
{
  buffer=RCREG; /* RCIF is cleared by hardware */
}

void main(void)
{
  unsigned char buffer2;

  portinit();
  uart_init();

  while(!RCIF) ; /* wait for first byte */
  GIE=1;         /* interrupts are actived */
  asm("nop");

  for(;;) {
    GIE=0;         /* interrupts are disabled */
    buffer2=buffer;
    GIE=1;         /* interrupts are actived */
    display(buffer2);
  }
}
