/*============================================================================

           EduKitBeta & MU Beta example : Alarm clock

 Compiler/IDE : PICC-Lite and IDEA

 Processor : PIC16F627
 Clock : XT 3.2768 MHz

 On your EduKitBeta board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on A4 LED on S3
 3) Switch on B3 BUZZER on S4
 4) Switch on B4 DISPLAY on S4

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & LVPDIS & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

typedef unsigned int clock_t;

static bit led @ PORTBIT(PORTA,4);
static bit data @ PORTBIT(PORTB,7); /* shift register data */
static bit clk @ PORTBIT(PORTB,4); /* shift register clock */
static bit str @ PORTBIT(PORTB,6); /* shift register strobe */
static bit pushbutton2 @ PORTBIT(PORTB,7);
static bit pushbutton3 @ PORTBIT(PORTA,4);

char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t lap1,lap2;
unsigned int period1,period2;

unsigned char time_hrs=0; /* real time */
unsigned char time_min=0;
unsigned char alarm_hrs=12; /* alarm time */
unsigned char alarm_min=0;

char dstat=0;
char alarmtime=0;
volatile unsigned char buttonsbuffer=0x00;
volatile char blink=0;

/*============================================================================

       Program

============================================================================*/

void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=1; PS1=0; PS0=1; /* select prescaler by 64 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}

void timer1init(void)
{
 /* initialisation of timer1 */
	T1CKPS1=1; T1CKPS0=0; /* select prescaler by 4 */
	TMR1CS=0; /* select clock source */
	TMR1ON=1;  /* select prescaler for timer0 */
}

void pwminit(void)
{
  TMR2=0;
  TMR2ON=1;
  PR2=0xFF;
  CCPR1H=0;
  CCPR1L=0;
  CCP1CON=0x0F; /* PWM mode */
}

unsigned char getbb(void)
{ /* return buttonsbuffer */
  unsigned char tmp;

  GIE=0;
  tmp=buttonsbuffer;
  GIE=1;
  return tmp;
}

void buttons_wait(void) /* wait for release all buttons */
{
  while(getbb()) ;
}

void sendbyte(unsigned char b)
{
  char i;
  char tmp;

  tmp=b;
  for(i=8;i;i--) {
    if(tmp&0x80)
      data=1;
    else
      data=0;
    clk=0;
    asm("nop");
    clk=1;
    tmp<<=1;
  }
}

void display(unsigned char disdt[])
{
  char n;

  str=0;
  for(n=4;n;n--) {
    sendbyte(disdt[n-1]);
  }
  str=1;
}

void settime(unsigned char *p_time, char max)
{
  GIE=0;
  if((++(*p_time))>max)
    *p_time=0;
  GIE=1;
}

void alarm(void)
{
  unsigned char count;

  for(count=35;count;count--) {
    CCPR1L=0x80; /* PWM on */
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) { /* wait for timer 1 overflow */
      if(getbb()) { /* test buttons */
        buttons_wait(); /* wait for release */
        CCPR1L=0x00; /* PWM off */
        return;
      }
    }
    CCPR1L=0x00; /* PWM off */
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) { /* wait for timer 1 overflow */
      if(getbb()) { /* test buttons */
        buttons_wait(); /* wait for release */
        return;
      }
    }
  }
}

void interrupt timer0(void)
{
  unsigned char distmp[4];

  systimer++;
  T0IF=0;
  if ((systimer-lap1)>period1) {
    lap1+=period1;
    if(++time_min>=60) { /* increment minutes */
      time_min=0;
      if(++time_hrs>=24)  /* increment hours */
        time_hrs=0;
    }
    if((time_min==alarm_min)&(time_hrs==alarm_hrs))
      alarmtime=1;
  }
  TRISB|=0x80; /* RB7 is input */
  TRISA|=0x10; /* RA4 is input */
  asm("nop");
  buttonsbuffer=0;
  if(!pushbutton2)
    buttonsbuffer+=1;
  if(!pushbutton3)
    buttonsbuffer+=2;
  TRISB&=0x7F; /* RB7 is output */
  TRISA&=0xEF; /* RA4 is output */

  if ((systimer-lap2)>period2) {
    lap2+=period2;
    blink=!blink;
    switch(dstat) {
      case(0) :          /* show real time */
        distmp[3]=segment[time_hrs/10];
        distmp[2]=segment[time_hrs%10];
        distmp[1]=segment[time_min/10];
        distmp[0]=segment[time_min%10];
        if(blink)
          distmp[2]&=0x7F; /* blinking dot */
        led=1;
        break;

      case(1) :          /* show alarm time */
        distmp[3]=segment[alarm_hrs/10];
        distmp[2]=segment[alarm_hrs%10];
        distmp[1]=segment[alarm_min/10];
        distmp[0]=segment[alarm_min%10];
        led=0;
        break;

      case(2) :          /* show setup real time - hours */
        distmp[3]=segment[time_hrs/10];
        distmp[2]=segment[time_hrs%10]&0x7F;
        if(blink) {
          distmp[3]=0xFF;
          distmp[2]=0xFF;
        }
        distmp[1]=segment[time_min/10];
        distmp[0]=segment[time_min%10];
        led=1;
        break;

      case(3) :          /* show setup real time - minutes */
        distmp[3]=segment[time_hrs/10];
        distmp[2]=segment[time_hrs%10];
        distmp[1]=segment[time_min/10];
        distmp[0]=segment[time_min%10];
        if(blink) {
          distmp[1]=0xFF;
          distmp[0]=0xFF;
          distmp[2]&=0x7F; /* blinking dot */
        }
        led=1;
        break;

      case(4) :          /* show setup alarm time - hours */
        distmp[3]=segment[alarm_hrs/10];
        distmp[2]=segment[alarm_hrs%10]&0x7F;
        if(blink) {
          distmp[3]=0xFF;
          distmp[2]=0xFF;
        }
        distmp[1]=segment[alarm_min/10];
        distmp[0]=segment[alarm_min%10];
        led=0;
        break;

      case(5) :          /* show setup alarm time - minutes */
        distmp[3]=segment[alarm_hrs/10];
        distmp[2]=segment[alarm_hrs%10]&0x7F;
        distmp[1]=segment[alarm_min/10];
        distmp[0]=segment[alarm_min%10];
        if(blink) {
          distmp[1]=0xFF;
          distmp[0]=0xFF;
        }
        led=0;
        break;
      
    } /* end of switch statement */
    display(distmp);
  }
}
 
int main(void)
{
  PORTB=0x00;
  PORTA=0x10;
  TRISA=0xEF;
  TRISB=0x27;
  timer0init();
  timer1init();
  pwminit();

  period1=3000;    /* 20ms*3000 = 1 minute */
  period2=25;   /* 20ms*25 = 500ms - blinking dot period */
  GIE=1;

  for(;;)
  {
    if(getbb()&1) {
      switch(dstat) {
        case (1) :
        case (5) :
          dstat=0;
          break;
        default :
          dstat++;
          break;
      }
      buttons_wait();
    }

    if(getbb()&2) {
      switch(dstat) {
        case (0) :
        case (1) :
          dstat=2;
          break;
        case (2) :
          settime(&time_hrs,23);
          break;
        case (3) :
          settime(&time_min,59);
          break;
        case (4) :
          settime(&alarm_hrs,23);
          break;
        case (5) :
          settime(&alarm_min,59);
          break;
      }
      buttons_wait();
    }

    if(alarmtime) {
      alarmtime=0;
      alarm();
    } 
  }
}
